More limp home handling
[revag-nm.git] / vw-nm.c
diff --git a/vw-nm.c b/vw-nm.c
index 50863aac7e8fde2c932163aa927ffc48eaaa8f89..5afa9612738d6da7cb23bb10f9bfdce717e767e1 100644 (file)
--- a/vw-nm.c
+++ b/vw-nm.c
@@ -60,11 +60,20 @@ static void nm_update_my_next_id(struct NM_Main *nm) {
                state = nm->nodes[id].state & NM_MAIN_MASK;
 
                if (state == NM_MAIN_ON || state == NM_MAIN_LOGIN) {
-                       /* TODO: Check for limp home nodes? */
+                       /* We skip limp home nodes */
                        nm->nodes[nm->my_id].next = id;
                        break;
                }
        } while (id != nm->my_id);
+
+       if (nm->nodes[nm->my_id].next == nm->my_id) {
+               /* Uh oh, we're the only one left. */
+
+               /* TODO */
+               nm->nodes[nm->my_id].state = NM_MAIN_LOGIN;
+
+               /* TODO: Timeout 140ms (RCD 310) */
+       }
 }
 
 
@@ -108,21 +117,27 @@ static void nm_handle_can_frame(struct NM_Main *nm, struct can_frame *frame)
                                nm->tv.tv_sec = 0;
                                nm->tv.tv_usec = 0;
                                /* IMPORTANT: The caller needs to check for
-                                * timeouts first, so no other NM frames are
-                                * received until our correcting login has
-                                * been sent.
+                                * timeouts first, i.e. no other NM frames
+                                * are received until our correcting login
+                                * has been sent.
                                 */
                        } else if (next == nm->nodes[nm->my_id].next) {
-                               /* where nm->nodes[nm->my_id].next == nm->my_id */
-
-                               /* TODO: Is this a case we need to handle? */
+                               /* where (nm->nodes[nm->my_id].next == nm->my_id) */
 
                                /* It can happen when:
                                 *  - our sent frames don't go anywhere
                                 *  - we just logged in and immediately
                                 *    afterwards another ECU sent a regular
-                                *    NM frame without knowing that we exist.
+                                *    NM frame.
+                                */
+
+                               /* Let's handle this just like a LOGIN, since
+                                * we're learning about a new device.
+                                * See case NM_MAIN_LOGIN below for details.
                                 */
+
+                               nm_update_my_next_id(nm);
+                               nm->nodes[nm->my_id].state = NM_MAIN_ON;
                        } else if (next == nm->my_id) {
                                /* It's our turn.
                                 * Reset the timeout so anyone we missed
@@ -136,29 +151,32 @@ static void nm_handle_can_frame(struct NM_Main *nm, struct can_frame *frame)
                                 * connectivity.
                                 */
                                nm->tv.tv_sec = 0;
-                               nm->tv.tv_usec = NM_USECS_LIMP_HOME;
+                               nm->tv.tv_usec = NM_USECS_NODE_AWOL;
                        }
                        break;
                case NM_MAIN_LOGIN:
                        /* Note: sender != nm->my_id */
 
-                       printf("Handling LOGIN\n");
-
                        nm_update_my_next_id(nm);
 
                        /* We're not alone anymore, so let's change state. */
                        nm->nodes[nm->my_id].state = NM_MAIN_ON;
 
/* We don't reset the timeout when somebody logs in.
                         * Instead, we'll simply include them in the next
                         * round. */
 
                        /* Actually, when a login is done as a correction,
                         * we do reset the timeout.
-                        *
-                        * TODO.
                         */
+                       if (next != sender) {
+                               nm->tv.tv_sec = 0;
+                               nm->tv.tv_usec = NM_USECS_NODE_AWOL;
+                       }
                        break;
+               case NM_MAIN_LIMPHOME:
+                       nm_update_my_next_id(nm);
        }
 
        nm_dump_all(nm);
@@ -183,7 +201,7 @@ static void nm_buildframe(struct NM_Main *nm, struct can_frame *frame)
 static void nm_timeout_callback(struct NM_Main *nm, struct can_frame *frame)
 {
        nm->tv.tv_sec = 0;
-       nm->tv.tv_usec = NM_USECS_LIMP_HOME;
+       nm->tv.tv_usec = NM_USECS_NODE_AWOL;
 
        nm_buildframe(nm, frame);
 }
@@ -136,29 +151,32 @@ static void nm_handle_can_frame(struct NM_Main *nm, struct can_frame *frame)
                        nm->nodes[nm->my_id].state = NM_MAIN_ON;
 
-                       /* We don't reset the timeout when somebody logs in.
+                       /* We don't reset the timeout when somebody logs in,
+                        * i.e. (next == sender).
                         * Instead, we'll simply include them in the next
                         * round. */